package com.trust.ywx.customer.service.manager.impl;

import com.fiftyonetrust.common.model.exception.CodeException;
import com.trust.ywx.customer.common.enu.*;
import com.trust.ywx.customer.common.model.manager.request.TempFirmParam;
import com.trust.ywx.customer.common.model.manager.request.FirmParam;
import com.trust.ywx.customer.commons.UniqueIdUtils;
import com.trust.ywx.customer.dao.TempFirmDao;
import com.trust.ywx.customer.dao.po.FirmInfoTemp;
import com.trust.ywx.customer.service.manager.FirmManagerService;
import com.trust.ywx.customer.service.manager.TempFirmManagerService;
import lombok.SneakyThrows;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.stereotype.Service;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;

/**
 * <pre></pre>
 *
 * @文件名称： FirmManagerServiceImpl.java
 * @包 路  径： com.trust.ywx.customer.service.manager.impl
 * @Copyright：北京数字医信责任有限公司 (C) 2022 *
 * @Description: 客户管理实现类
 * @Version: V1.0
 * @Author： hanzhenchao
 * @Date：2022/6/20 12:50
 * @Modify：
 */
@Service
public class TempFirmManagerServiceImpl implements TempFirmManagerService {
    @Resource
    TransactionTemplate transactionTemplate;
    @Resource
    TempFirmDao tempFirmDao;
    @Resource
    FirmManagerService firmManagerService;
    @Value("${firm.mss_app_id}")
    private String mssAppId;


    @Override
    public String add(TempFirmParam param) throws CodeException {
        // 参数校验
        checkParam(param, true);
        // 业务校验
        checkBiz(param, true);
        // 入库
        return doAddTemp(param);
    }

    @Override
    @CacheEvict(cacheNames = "firm", key = "'temp:'+#param.clientId")
    public boolean update(TempFirmParam param) throws CodeException {
        // 参数校验
        checkParam(param, false);
        // 业务校验
        checkBiz(param, false);
        // 入库
        return doUpdateTemp(param);
    }

    @Override
    public boolean audit(String clientId) throws CodeException {
        // 校验clientId
        FirmInfoTemp tempFirm = tempFirmDao.getByClientId(clientId);
        if (null == tempFirm) {
            throw new CodeException(FirmErrEum.FIRM_NOT_EXIST.getStatus(),FirmErrEum.FIRM_NOT_EXIST.getMessage());
        }
        // 封装可信客户参数
        FirmParam firmParam = buildTrustFirmParam(tempFirm);

        return doAudit(firmParam, tempFirm);
    }


    @Override
    public boolean reject(String clientId) throws CodeException {
        // 业务校验
        FirmInfoTemp tempFirm = tempFirmDao.getByClientId(clientId);
        if (null == tempFirm) {
            throw new CodeException(FirmErrEum.CLIENTID_CHECK_ERR.getStatus(),FirmErrEum.CLIENTID_CHECK_ERR.getMessage());
        }

        int num = tempFirmDao.auditReject(clientId);
        return num == 1;
    }

    @Override
    public boolean delete(String clientId) {
        // fixme 此处应该为物理删除，否则客户在此添加时 客户名称就重复了
        tempFirmDao.deleteByClientId(clientId);
        return true;
    }

    private boolean doUpdateTemp(TempFirmParam param) {
        FirmInfoTemp temp = convertToTempFirm(param, false);
        return tempFirmDao.update(temp) == 1;
    }

    private void checkBiz(TempFirmParam param, boolean isAdd) throws CodeException {
        // clientId
        if (!isAdd) {
            FirmInfoTemp temp = tempFirmDao.getByClientId(param.getClientId());
            if (null == temp) {
                throw new CodeException(FirmErrEum.FIRM_NOT_EXIST.getStatus(),FirmErrEum.FIRM_NOT_EXIST.getMessage());
            }
        }
        // firmName冲突校验
        List<FirmInfoTemp> list = tempFirmDao.getListByFirmName(param.getFirmName());
        if (null != list && list.size() > 0) {
            if (isAdd) {
                throw new CodeException(FirmErrEum.FIRM_NAME_CHECK_ERR.getStatus(),FirmErrEum.FIRM_NAME_CHECK_ERR.getMessage());
            } else {
                for (FirmInfoTemp temp : list) {
                    if (!Objects.equals(temp.getClientId(), param.getClientId())) {
                        throw new CodeException(FirmErrEum.FIRM_NAME_CHECK_ERR.getStatus(),FirmErrEum.FIRM_NAME_CHECK_ERR.getMessage());
                    }
                }
            }
        }
    }

    private String doAddTemp(TempFirmParam param) {
        FirmInfoTemp temp = convertToTempFirm(param, true);
        tempFirmDao.insert(temp);
        return temp.getClientId();

    }

    private FirmInfoTemp convertToTempFirm(TempFirmParam param, boolean isAdd) {
        FirmInfoTemp temp = new FirmInfoTemp();
        BeanUtils.copyProperties(param, temp);
        if (isAdd) {
            temp.setUniqueid(UniqueIdUtils.genrateUniqueId());
            temp.setClientId(UniqueIdUtils.generateClientId());
        }
        temp.setFirmOrgCode(param.getFirmOrgCode());
        temp.setStatus(TempFirmStatusEnum.TEMP_UNAUTHEN.getCode() + "");
        if (param.getChannelApp() != null && param.getChannelApp() == 1
                && param.getChannelSdk() != null && param.getChannelSdk() == 1) {
            temp.setChannel("2");
        } else if (param.getChannelSdk() != null && param.getChannelSdk() == 1) {
            temp.setChannel("1"); //sdk
        } else if (param.getChannelApp() != null && param.getChannelApp() == 1) {
            temp.setChannel("0"); //app
        }
        return temp;

    }

    private boolean doAudit(FirmParam firmParam, FirmInfoTemp temp) {
        transactionTemplate.execute(new TransactionCallbackWithoutResult() {
            @SneakyThrows
            @Override
            protected void doInTransactionWithoutResult(TransactionStatus transactionStatus) {
                // 1、添加可信客户
                firmManagerService.add(firmParam);
                // 2、更新临时客户状态为已审核
                int num = tempFirmDao.auditPass(temp.getClientId());
                if (num != 1) {
                    throw new CodeException(FirmErrEum.FIRM_SYN_ERR.getStatus(),FirmErrEum.FIRM_SYN_ERR.getMessage());
                }
            }
        });
        return true;
    }

    private FirmParam buildTrustFirmParam(FirmInfoTemp temp) {
        FirmParam firmParam = new FirmParam();
        BeanUtils.copyProperties(temp, firmParam);
        firmParam.setAppIcon(temp.getFirmLogo());
        // 医师端产品形态
        if ("0".equals(temp.getChannel())) {
            firmParam.setDoctorProductApp(CommonStatusEnum.YES.getCode());
        } else if ("1".equals(temp.getChannel())) {
            firmParam.setDoctorProductSdk(CommonStatusEnum.YES.getCode());
        } else if ("2".equals(temp.getChannel())) {
            firmParam.setDoctorProductApp(CommonStatusEnum.YES.getCode());
            firmParam.setDoctorProductSdk(CommonStatusEnum.YES.getCode());
        }
        firmParam.setCertAlg(CertAlgEnum.SM2.getName());
        String clientId = null;
        if (StringUtils.isBlank(temp.getClientId())) {
            // fixme 设么时候会出现这种情况？
            clientId = UniqueIdUtils.generateClientId();
        } else {
            clientId = temp.getClientId();
        }
        firmParam.setClientId(clientId);
        firmParam.setAppSecret(UniqueIdUtils.generateAppSecret());
        // 信步云证书ID 当医师产品形态为空、APP、APP+SDK 都为托管的mssAppId
        if (StringUtils.isBlank(temp.getChannel()) || "1".equals(temp.getChannel()) || "2".equals(temp.getChannel())) {
            firmParam.setMsspAppid(mssAppId);
        }
        /**
         * 填充默认项
         */
//        // 自动设置图章 ：是
//        firmParam.setUserStampAutoSet(CommonStatusEnum.YES.getCode());
//        // 审核完发送短信 ：是
//        firmParam.setSmsSend(CommonStatusEnum.YES.getCode());
//        // 证书发放厂家:信步云
//        firmParam.setCertAgency("1");
//        // 渠道：北京CA
//        firmParam.setCaChannel(CaChannelEnum.BJCA.getChannel());
//        // 客户类型：互联网医院
//        firmParam.setNetworkType(NetworkTypeEnum.IT.getCode());
//        // app设置图章：可以
//        firmParam.setUserStampIsAppset(CommonStatusEnum.YES.getCode());
//        // 客户图章类型：圆章
//        firmParam.setFirmStampType(FirmStampEnum.ROUND.getCode());

        return firmParam;
    }


    /**
     * 参数校验
     *
     * @param param
     * @param isAdd
     */
    private void checkParam(TempFirmParam param, boolean isAdd) throws CodeException {
        if (!isAdd && StringUtils.isBlank(param.getClientId())) {
            throw new CodeException(FirmErrEum.FIRM_CLIENT_ID_NULL.getStatus(),FirmErrEum.FIRM_CLIENT_ID_NULL.getMessage());
        }
        if (StringUtils.isBlank(param.getFirmName())) {
            throw new CodeException(FirmErrEum.FIRM_NAME_NULL.getStatus(),FirmErrEum.FIRM_NAME_NULL.getMessage());
        }
        if (StringUtils.isNotEmpty(param.getFirmName()) && param.getFirmName().length() > 30) {
            throw new CodeException(FirmErrEum.FIRM_NAME_TOO_LONG.getStatus(),FirmErrEum.FIRM_NAME_TOO_LONG.getMessage());
        }

    }


}
